@@ -13,15 +13,18 @@
 #ifndef OPENVDB_TOOLS_VOLUME_ADVECT_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_VOLUME_ADVECT_HAS_BEEN_INCLUDED
 
-#include <tbb/parallel_for.h>
-#include <openvdb/Types.h>
-#include <openvdb/math/Math.h>
-#include <openvdb/util/NullInterrupter.h>
+#include "openvdb/Types.h"
+#include "openvdb/math/Math.h"
+#include "openvdb/util/NullInterrupter.h"
+#include "openvdb/thread/Threading.h"
 #include "Interpolation.h"// for Sampler
 #include "VelocityFields.h" // for VelocityIntegrator
 #include "Morphology.h"//for dilateActiveValues
 #include "Prune.h"// for prune
 #include "Statistics.h" // for extrema
+
+#include <tbb/parallel_for.h>
+
 #include <functional>
 
 
@@ -33,7 +36,7 @@ namespace tools {
 namespace Scheme {
     /// @brief Numerical advections schemes.
     enum SemiLagrangian { SEMI, MID, RK3, RK4, MAC, BFECC };
-    /// @brief Flux-limiters employed to stabalize the second-order
+    /// @brief Flux-limiters employed to stabilize the second-order
     /// advection schemes MacCormack and BFECC.
     enum Limiter { NO_LIMITER, CLAMP, REVERT };
 }
@@ -46,7 +49,7 @@ namespace Scheme {
 ///
 /// @note  Optionally a limiter can be combined with the higher-order
 ///        integration schemes MacCormack and BFECC. There are two
-///        types of limiters (CLAMP and REVERT) that supress
+///        types of limiters (CLAMP and REVERT) that suppress
 ///        non-physical oscillations by means of either claminging or
 ///        reverting to a first-order schemes when the function is not
 ///        bounded by the cell values used for tri-linear interpolation.
@@ -198,7 +201,7 @@ public:
     ///
     /// @warning If the VolumeSamplerT is of higher order than one
     ///          (i.e. tri-linear interpolation) instabilities are
-    ///          known to occure. To suppress those monotonicity
+    ///          known to occur. To suppress those monotonicity
     ///          constrains or flux-limiters need to be applies.
     ///
     /// @throw RuntimeError if @a inGrid does not have uniform voxels.
@@ -243,7 +246,7 @@ public:
     ///
     /// @warning If the VolumeSamplerT is of higher order the one
     ///          (i.e. tri-linear interpolation) instabilities are
-    ///          known to occure. To suppress those monotonicity
+    ///          known to occur. To suppress those monotonicity
     ///          constrains or flux-limiters need to be applies.
     ///
     /// @throw RuntimeError if @a inGrid is not aligned with @a mask
@@ -294,7 +297,7 @@ private:
     bool interrupt() const
     {
         if (mInterrupter && util::wasInterrupted(mInterrupter)) {
-            tbb::task::self().cancel_group_execution();
+            thread::cancelGroupExecution();
             return true;
         }
         return false;
@@ -534,6 +537,32 @@ struct VolumeAdvection<VelocityGridT, StaggeredVelocity, InterrupterType>::Advec
     const VolumeAdvection*    mParent;
 };// end of private member class Advect
 
+
+////////////////////////////////////////
+
+
+// Explicit Template Instantiation
+
+#ifdef OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+#ifdef OPENVDB_INSTANTIATE_VOLUMEADVECT
+#include <openvdb/util/ExplicitInstantiation.h>
+#endif
+
+OPENVDB_INSTANTIATE_CLASS VolumeAdvection<Vec3SGrid, true, util::NullInterrupter>;
+OPENVDB_INSTANTIATE_CLASS VolumeAdvection<Vec3SGrid, false, util::NullInterrupter>;
+
+OPENVDB_INSTANTIATE FloatGrid::Ptr VolumeAdvection<Vec3SGrid, true, util::NullInterrupter>::advect<FloatGrid, Sampler<1, false>>(const FloatGrid&, double);
+OPENVDB_INSTANTIATE DoubleGrid::Ptr VolumeAdvection<Vec3SGrid, true, util::NullInterrupter>::advect<DoubleGrid, Sampler<1, false>>(const DoubleGrid&, double);
+OPENVDB_INSTANTIATE Vec3SGrid::Ptr VolumeAdvection<Vec3SGrid, true, util::NullInterrupter>::advect<Vec3SGrid, Sampler<1, false>>(const Vec3SGrid&, double);
+
+OPENVDB_INSTANTIATE FloatGrid::Ptr VolumeAdvection<Vec3SGrid, false, util::NullInterrupter>::advect<FloatGrid, Sampler<1, false>>(const FloatGrid&, double);
+OPENVDB_INSTANTIATE DoubleGrid::Ptr VolumeAdvection<Vec3SGrid, false, util::NullInterrupter>::advect<DoubleGrid, Sampler<1, false>>(const DoubleGrid&, double);
+OPENVDB_INSTANTIATE Vec3SGrid::Ptr VolumeAdvection<Vec3SGrid, false, util::NullInterrupter>::advect<Vec3SGrid, Sampler<1, false>>(const Vec3SGrid&, double);
+
+#endif // OPENVDB_USE_EXPLICIT_INSTANTIATION
+
+
 } // namespace tools
 } // namespace OPENVDB_VERSION_NAME
 } // namespace openvdb
